<?php

/**
 * @file theme_registry.alter.inc
 * Alters the theme registry.
 */
// Calls hex_file_scan()
include_once HEX_INCLUDES . '/build_files.inc';

/**
 * Anything containing a theme function or variable process function must be
 * included here in order for it to be discovered.
 */
foreach (hex_get_theme_registry_files() as $file) {
  switch ($file->type) {
    case 'theme':
    case 'func' :
      include_once DRUPAL_ROOT . '/' . $file->uri;
  }
}

/**
 * implements hook_theme_registry_alter()
 */
function hex_theme_registry_hex_alter(&$registry) {

  // hook_theme() for plug-ins.
  foreach (hex_plugin_list() as $plugin) {
    _theme_process_registry($registry, $plugin, 'plugin', $plugin, hex_plugin_path($plugin));
  }

  $includes = array();
  foreach (hex_get_theme_registry_files() as $file) {
    switch ($file->type) {
      case 'theme':
      case 'func' :
      case 'vars' :
        $includes[$file->name][] = $file->uri;
    }
  }

  $active_theme = hex_active_theme();
  $include_vars = hex_path('theme', 'hex') . '/includes/include_vars.inc';

  foreach ($registry as $theme_hook => &$hook_info) {

    // build hook info.
    $hex_info = hex_hook_info($registry, $theme_hook, $hook_info);

    // Initialize. Make sure these elements are available.
    // Empty elements will be removed later.
    $hook_includes = &$hook_info['includes'];
    $preprocesses  = &$hook_info['preprocess functions'];
    $processes     = &$hook_info['process functions'];
    foreach (array('includes', 'preprocess functions', 'process functions') as $key) {
      if (!isset($hook_info[$key])) {
        $hook_info[$key] = array();
      }
    }

    // Add HOOK_SOURCE.theme.inc files into Core's ability to include them on
    // the fly through theme() calls. The hook source is which ever module,
    // theme or plug-in that originally implements the theming hook.
    //
    // Process:
    // - [ MODULE | THEME | PLUGIN ].theme.inc

    if ($hex_info->implement_level !== 'module') {
      // Multiple sources since hook patterns can be defined separately from base hook.
      foreach ($hex_info->sources as $source) {
        if (isset($includes["$source.theme"]) && !in_array("$source.theme", $hook_includes)) {
          $hook_includes = array_merge($hook_includes, $includes["$source.theme"]);
        }
      }
    }

    // ->hooks can contain both the base hook and pattern hook.
    foreach ($hex_info->hooks as $hook) {
      $hook_name    = str_replace('_', '-', $hook);

      // Process:
      // - [ HOOK ].func.inc
      // - [ HOOK--PATTERN ].func.inc

      if ($hex_info->implement_level === 'theme'
      &&  $hex_info->implement_type  === 'func'
      &&  isset($includes["$hook_name.func"])) {
        // Use last element which will be in the child-most theme.
        $hook_includes[] = end($includes["$hook_name.func"]);
      }

      // The file will host variables specific to the theming hook whether it
      // is implemented as a function or template. Some are more focused than
      // others.
      //
      // Process:
      // - [ HOOK ].vars.inc
      // - [ HOOK ].[ ACTIVE-THEME ].vars.inc
      // - [ HOOK ].[ func | tpl ].vars.inc
      // - [ HOOK ].[ ACTIVE-THEME ].[ func | tpl ].vars.inc
      // - [ HOOK--PATTERN ].vars.inc
      // - [ HOOK--PATTERN ].[ ACTIVE-THEME ].vars.inc
      // - [ HOOK--PATTERN ].[ func | tpl ].vars.inc
      // - [ HOOK--PATTERN ].[ ACTIVE-THEME ].[ func | tpl ].vars.inc

      foreach (array($hook_name,
                    "$hook_name.$active_theme",
                    "$hook_name.{$hex_info->implement_type}",
                    "$hook_name.$active_theme.{$hex_info->implement_type}") as $named) {

        if (isset($includes["$named.vars"])) {

          // Include the file that handles hook.vars.inc files.
          if (!in_array($include_vars, $hook_includes)) {
            $hook_includes[] = $include_vars;
          }

          // Move to the front of second phase process so it always comes after
          // preprocess functions but before process functions.

          if (!in_array('hex_variable_include_process', $processes)) {
            array_unshift($processes, 'hex_variable_include_process');
          }
          // Add path to allow the custom preprocessor to load the file.
          if (!isset($hex_info->hex_variable_includes)) {
            $hex_info->variable_includes = array();
          }
          $hex_info->variable_includes = array_merge($hex_info->variable_includes, $includes["$named.vars"]);
        }
      }

      // Add in variable process functions for plugins.
      if (hex_plugin_list()) {
        $process_functions = array(
          'preprocess' => &$preprocesses, 
          'process'    => &$processes,
        );
        foreach ($process_functions as $phase => &$phase_process) {
          foreach (hex_plugin_list() as $prefix) {
            $var_func = $prefix . '_' . $phase;
            if (isset($hook_info['template'])
            && function_exists($var_func)
            && !in_array($var_func, $phase_process)) {
              // PLUGIN_preprocess() and PLUGIN_process() pushed up front.
              array_unshift($phase_process, $var_func);
            }
            // The rest come last.
            $var_hook_functions = array(
              $var_func . '_' . $hook,
              $var_func . '_' . $active_theme . '_' . $hook,
            );
            foreach ($var_hook_functions as $var_func) {
              if (function_exists($var_func)
              && !in_array($var_func, $phase_process)) {
                $phase_process[] = $var_func;
              }
            }
          }
        }
      }
    }

    // Optimize the registry.
    foreach (array('includes', 'preprocess functions', 'process functions') as $key) {
      if (empty($hook_info[$key])) {
        unset($hook_info[$key]);
      }
    }

    // Add hex info to the registry.
    $hook_info['hex_info'] = $hex_info;
  }
}

/**
 * Builds info about a theming hook for the theme registry.
 */
function hex_hook_info($registry, $theme_hook, $hook_info) {

  $implement_levels = array(
    'module'            => 'module',
    'base_theme_engine' => 'theme',
    'base_theme'        => 'theme',
    'theme_engine'      => 'theme',
    'theme'             => 'theme',
    'plugin'            => 'plugin',
  );

  $hex_info = new stdClass();
  $hex_info->implement_level = $implement_levels[$hook_info['type']];
  $hex_info->implement_type  = isset($hook_info['function']) ? 'func' : 'tpl';
  $hex_info->element_type    = NULL;
  if (isset($hook_info['render element'])) {
    $hex_info->element_type = $hook_info['render element'];
  }

  $hook_sources = hex_get_theme_hook_sources();

  // Multiple hooks used since this accounts for hook__suggestions.
  $hex_info->hooks = array($theme_hook);

  // Identify the source of the hook.
  $_hook = isset($hook_info['base hook']) ? $hook_info['base hook'] : $theme_hook;
  $hex_info->sources = array($hook_sources[$_hook]);

  // Do the same looking through named patterns.
  $hook = $theme_hook;
  while ($pos = strrpos($hook, '__')) {
    $hook = substr($hook, 0, $pos);
    if (isset($registry[$hook])) {

      $hex_info->hooks[] = $hook;

      $_hook = isset($registry[$hook]['base hook']) ? $registry[$hook]['base hook'] : $hook;
      if (!in_array($hook_sources[$_hook], $hex_info->sources)) {
        $hex_info->sources[] = $hook_sources[$_hook];
      }

    }
  }

  // Look for the current host.
  if ($hex_info->implement_level === 'module') {
    $hex_info->host = $hex_info->sources[0];
  }
  else {
    foreach (hex_list($hex_info->implement_level) as $name) {
      if ($hook_info['theme path'] === hex_path($hex_info->implement_level, $name)) {
        $hex_info->host = $name;
        break;
      }
    }
  }

  return $hex_info;
}

/**
 * Scans for files needed for the theme registry.
 */
function hex_get_theme_registry_files() {
  $cache = &drupal_static(__FUNCTION__, NULL);

  if (!isset($cache)) {
    $cache = array();
    $hooks_sources = hex_get_theme_hook_sources();

    foreach (hex_file_scan('/^[[:alpha:]][[:alpha:]._-]+\.(theme|func|vars)\.inc$/') as $files_data) {
      foreach ($files_data as $file_data) {
        $parts = explode('.', $file_data->name);
        $name  = array_shift($parts);
        $type  = array_pop($parts);

        if ($type === 'theme' && in_array($name, $hooks_sources)
        ||  $type === 'func'  && isset($hooks_sources[$name])
        ||  $type === 'vars'  && isset($hooks_sources[$name])) {

          $file_data->basename = $name;
          $file_data->type     = $type;
          $cache[] = $file_data;
        }
      }
    }
  }

  return $cache;
}

/**
 * Returns all theming hooks and their sources.
 */
function hex_get_theme_hook_sources() {
  $cache = &drupal_static(__FUNCTION__, NULL);

  if (!isset($cache)) {
    $cache = array();

    // False pass of hook_theme() so the theming hooks implemented by each
    // component can be determined. $theme_hooks does nothing. This is only
    // to prevent potential notices or errors.

    $theme_hooks = array();
    foreach (array('module' => hex_list('module'),
                   'theme'  => hex_list('theme'),
                   'plugin' => hex_list('plugin')) as $type => $sources) {

      foreach ($sources as $source_key) {
        $hook_theme = $source_key . '_theme';
        if (function_exists($hook_theme)) {
          $result = $hook_theme($theme_hooks, $type, $source_key, hex_path($type, $source_key));
          foreach (array_keys($result) as $hook) {
            // Maintain first source.
            if (!isset($cache[$hook])) {
              $cache[$hook] = $source_key;
            }
          }
          $theme_hooks += $result;
        }
      }
    }
  }

  return $cache;
}
