<?php

/**
 * @file build_files.inc
 * Builds file paths. Included by Hexagon's cache system most of the time.
 */

/**
 * Build file paths based on file extensions and regex patterns set in the
 * .info file, .pinfo file or hook_file_patterns().
 *
 * The returned paths are keyed according to the search scope. Possible scopes
 * are "theme", "plugin", "all" or the name of the theme or plug-in hosting
 * the file.
 *
 * Patterns set from base themes are inherited into their children unless the
 * scope is named after a specific host. the same applies to plug-in patterns.
 * If the plug-in is hosted in a parent theme, the patterns defined in the
 * plug-in has the ability to find files of any child theme and its plug-ins.
 */
function hex_build_files() {

  $files = array();
  foreach (hex_active_themes() as $theme) {
    // Get plug-in defined files.
    foreach (hex_plugin_list("$theme.active") as $plugin) {
      _hex_build_files($files, $plugin, 'plugin');
    }
    // Get theme defined files.
    _hex_build_files($files, $theme, 'theme');
  }

  return $files;
}

/**
 * Private function. Does the heavy lifting for hex_build_files().
 */
function _hex_build_files(&$files, $name, $type) {

  $file_patterns = array();
  // .info/.pinfo defined extensions.
  foreach (call_user_func("hex_{$type}_info", 'file.extensions', array(), $name) as $group => $ext) {
    $ext = preg_quote($ext, '/');
    $file_patterns[$group][] = "/^[[:alpha:]][[:alnum:]._-]+$ext$/";
  }
  // .info/.pinfo defined patterns.
  foreach (call_user_func("hex_{$type}_info", 'file.patterns', array(), $name) as $group => $pattern) {
    $file_patterns[$group][] = $pattern;
  }
  // hook_file_patterns().
  if (function_exists($name . '_file_patterns')) {
    foreach (call_user_func($name . '_file_patterns', $file_patterns) as $group => $pattern) {
      $file_patterns[$group][] = $pattern;
    }
  }

  foreach ($file_patterns as $contexts => $patterns) {
    // Get search scope (prefixed) from pattern group.
    $search_scopes = array($type);
    if (!is_string($contexts)) {
      // Defaults to general group if not set.
      $group = 'general';
    }
    elseif (strpos($contexts, ':') !== FALSE) {
      // scope1|scope2:group
      list($scopes, $group) = explode(':', $contexts);
      $search_scopes = explode('|', $scopes);
    }
    foreach ($search_scopes as $scope) {
      foreach (hex_file_scan($patterns, $scope) as $found_files) {
        foreach ($found_files as $file) {
          $files[$scope][$group][$file->filename][] = $file;
          if ($scope !== 'all') {
            $files['all'][$group][$file->filename][] = $file;
          }
        }
      }
    }
  }
}

/**
 * Scans for files based on regex patterns and returns an array of paths.
 *
 * The scanning starts from base themes and their plug-ins to connected
 * sub-themes which are currently active.
 *
 * @param $mask
 *  Regex pattern to be compared with the name of the file. String or an array
 *  of strings are accepted.
 * @param $scope
 *  Can be 'theme', 'plugin' or 'all'. It can also be a plugin or theme name.
 *  whatever "owns" the file. File discovery depends on how the scope is
 *  defined in the group prefix and it must be accessed the same way with 'all'
 *  being the exception since it looks everywhere.
 */
function hex_file_scan($mask, $scope = 'all') {
  $cache     = &drupal_static(__FUNCTION__, array());
  $scan_data = &$cache[hex_active_theme()];

  if (!isset($scan_data)) {
    $scan_data = array();

    $theme_paths = $plugin_paths = $exclude_all_plugins = $exclude_inactive_plugins = array();

    foreach (list_themes() as $theme => $theme_data) {
      $theme_paths[$theme] = dirname($theme_data->filename);
    }
    foreach (hex_plugin_list('all') as $plugin) {
      $plugin_paths[$plugin] = hex_plugin_path($plugin);
    }
    $plugin_paths_inactive = $plugin_paths;
    foreach (hex_plugin_list('all.active') as $plugin) {
      unset($plugin_paths_inactive[$plugin]);
    }
    $exclude_all_plugins = array_values($plugin_paths);
    $exclude_inactive_plugins = array_values($plugin_paths_inactive);

    // This path will be cut down with each itteration in the following loop
    // to prevent unintentional scanning of unconnected themes and to force
    // a specific order while discovering files.
    $_exclude = $theme_paths;

    foreach (hex_active_themes() as $theme) {
      unset($_exclude[$theme]);

      // Get plug-in owned files first. This is important for cascading order.
      // Theme files have the final hand in overriding files so theme specific paths later.
      foreach (hex_plugin_list("$theme.active") as $plugin) {
        $scan                = new stdClass();
        $scan->hosts         = array($theme, $plugin);
        $scan->scope         = 'plugin';
        $scan->dir           = $plugin_paths[$plugin];
        $scan->excluded_from = $exclude_inactive_plugins;
        $scan_data[] = $scan;
      }

      // Get theme paths.
      $scan                = new stdClass();
      $scan->hosts         = array($theme);
      $scan->scope         = 'theme';
      $scan->dir           = $theme_paths[$theme];
      $scan->excluded_from = array_merge($exclude_all_plugins, array_values($_exclude));
      $scan_data[] = $scan;
    }
  }

  return _hex_file_scan($mask, $scan_data, $scope);
}

/**
 * @see hex_file_scan()
 */
function _hex_file_scan($masks, $scan_data, $scope = 'all') {
  $files = array();
  foreach ($scan_data as $scan) {
    if ($scope === 'all' || $scope === $scan->scope || in_array($scope, $scan->hosts)) {
      foreach ((array) $masks as $mask) {
        foreach (file_scan_directory($scan->dir, $mask) as $file) {
          if (strpos(str_replace($scan->dir, '', $file->uri), '/~') !== FALSE) {
            // Skip files and directories prefixed with a tilde relative to the scan
            // directory. Parent directories of the $scan->dir will not be affected.
            continue;
          }
          foreach ($scan->excluded_from as $exclude_path) {
            if (strpos($file->uri, "$exclude_path/") === 0) {
              // Skip to next file.
              continue 2;
            }
          }
          $file->hosts = $scan->hosts;
          $files[$file->filename][] = $file;
        }
      }
    }
  }

  return $files;
}
