<?php

define('HEX_VERSION', 2);

/**
 * @file build_plugins.inc
 * Builds plug-in data. Included by Hexagon's cache system.
 */
// Calls _hex_file_scan()
include_once HEX_INCLUDES . '/build_files.inc';
// Calls hex_themeapi_setting()
include_once HEX_INCLUDES . '/theme_api.inc';

/**
 * Returns plug-in data.
 */
function hex_build_plugins() {

  $persist_key = 'theme_' . hex_active_theme() . '_active_plugins';
  $persist_set = variable_get($persist_key, array());

  /* Build Plug-ins ------------------------------------------------------- */

  // Set defaults.
  $info_defaults = array(
    'name'                => 'Untitled',
    'description'         => '',
    'description.status'  => '',
    'stylesheets'         => array(),
    'scripts'             => array(),
    'plugin.type'         => 'undefined',
    'plugin.protected'    => 0,
    'plugin.dependencies' => array(),
    'module.dependencies' => array(),
    'dependencies'        => array(),
    'version'             => NULL,
    'php'                 => DRUPAL_MINIMUM_PHP,
  );

  foreach (list_themes() as $theme => $theme_data) {
    $theme_paths[$theme] = dirname($theme_data->filename);
  }

  // This path will be cut down with each iteration in the following loop
  // to prevent unintentional scanning of unconnected themes and to force
  // a specific order while discovering files.
  $_exclude_dir = $theme_paths;

  foreach (hex_active_themes() as $theme) {
    unset($_exclude_dir[$theme]);
    $scan                = new stdClass();
    $scan->hosts         = array($theme);
    $scan->scope         = 'theme';
    $scan->dir           = $theme_paths[$theme];
    $scan->excluded_from = $_exclude_dir;
    $scan_data[]         = $scan;
  }

  // Errors and warnings held to present them in the right order late in the
  // function. It is also used for various checks.
  $errors = $warnings = array();

  // Gather and build data of all available plug-in's held in each active theme.
  $plugins = array();
  foreach (_hex_file_scan('/^[[:alpha:]][[:alnum:]_]+\.pinfo$/', $scan_data) as $files) {
    $plugin            = $files[0];
    $plugin->directory = dirname($plugin->uri);
    $plugin->info      = drupal_parse_info_file($plugin->uri) + $info_defaults;
    if ($plugin->info['plugin.dependencies']) {
      $plugin->info['dependencies'] = $plugin->info['plugin.dependencies'];
    }

    // PHP compatibility.
    if (version_compare(phpversion(), $plugin->info['php']) < 0) {
      $error = new stdClass();
      $error->type    = 'php.incompatible';
      $error->phase   = 'build';
      $error->message = 'Theme plug-in <em>!this.name</em> is not compatible with PHP version <em>!server.php</em> installed on this server. Please upgrade to <em>!this.php</em>';
      $error->strings['!this.name']  = $plugin->info['name'];
      $error->strings['!this.php']   = $plugin->info['php'];
      $error->strings['!server.php'] = phpversion();
      $errors[$plugin->name][] = $error;
      continue;
    }

    // Base theme compatibility.
    if ((float) $plugin->info['version'] < HEX_VERSION) {
      $error = new stdClass();
      $error->type    = 'version.incompatible';
      $error->phase   = 'build';
      $error->message = 'Theme plug-in <em>!this.name</em> is not compatible with this version of Hexagon. Please upgrade to version <em>!hex.version</em>.';
      $error->strings['!this.name']   = $plugin->info['name'];
      $error->strings['!hex.version'] = HEX_VERSION;
      $errors[$plugin->name][] = $error;
      continue;
    }

    // Handle invalid state of having multiple copies in the same theme.
    if (count($files) > 1) {
      $paths = array();
      foreach ($files as $i => $file) {
        $paths[] = dirname(substr($file->uri, strlen(hex_path('theme', $file->hosts[0]))));
      }
      $error = new stdClass();
      $error->type    = 'duplicate';
      $error->phase   = 'build';
      $error->message = 'Theme plug-in <em>!this.name</em> has been blocked! Multiple copies found within <em>!host.name</em>: <em>!duplicates</em>.';
      $error->strings['!this.name']  = $plugin->info['name'];
      $error->strings['!host.name']  = hex_theme_info('name', 'Untitled', $plugin->hosts[0]);
      $error->strings['!duplicates'] = implode(', ', $paths);
      $errors[$plugin->name][] = $error;
      continue;
    }

    // If the plugin is protected, do not allow overriding.
    if (isset($plugins[$plugin->name]) && (int) $plugins[$plugin->name]->info['plugin.protected']) {
      $warning = new stdClass();
      $warning->type    = 'protected';
      $warning->phase   = 'build';
      $warning->message = 'Theme plug-in <em>!this.name</em> is protected. The override found within <em>!host.name</em> is not permitted.';
      $warning->strings['!this.name'] = $plugin->info['name'];
      $warning->strings['!host.name'] = hex_theme_info('name', 'Untitled', $plugin->hosts[0]);
      $warnings[$plugin->name][] = $warning;
      continue;
    }

    $plugins[$plugin->name] = $plugin;
  }

  if ($plugins) {
    // Alphabetical sort.
    ksort($plugins);

    // Check dependencies but do it incrementally.
    $plugin_by_host = array();
    // Set correct order.
    foreach (hex_active_themes() as $theme) {
      $plugin_by_host[$theme] = array();
    }
    // Fill out with references.
    foreach ($plugins as $plugin) {
      $plugin_by_host[$plugin->hosts[0]][$plugin->name] = &$plugins[$plugin->name];
    }

    $dependency_badorder = $dependency_errors = $dependency_missing = array();
    $accumulator = array();

    foreach ($plugin_by_host as $host => $host_plugins) {
      foreach (array_keys($host_plugins) as $key) {
        $accumulator[$key] = &$host_plugins[$key];
      }
      // Incrementally check for dependencies.
      // This core function is designed for modules but it's generic enough to
      // use for plug-in's. It builds dependency data with two new object
      // members, "requires" and "required_by".
      $accumulator = _module_build_dependencies($accumulator);

      // while (list($key, $plugin) = each($host_plugins)) {
      foreach ($host_plugins as $key => $plugin) {
        foreach (array_keys($plugin->requires) as $requires) {
          // Check for dependencies found so far. Plug-ins within base themes
          // pointing to other plug-ins hosted in a sub-theme is not be allowed.
          if (isset($plugins[$requires]) && !isset($accumulator[$requires])) {
            $dependency_badorder[$key][] = $requires;
          }
          // Catch bad order dependencies.
          elseif (isset($dependency_badorder[$requires])) {
            $dependency_errors[$key][] = $plugins[$requires]->info['name'];
          }
          // Error in dependency detected.
          elseif (isset($errors[$requires])) {
            $dependency_errors[$key][] = $errors[$requires][0]->strings['!this.name'];
          }
          // Missing dependency.
          elseif (!isset($plugins[$requires])) {
            $dependency_missing[$key][] = $requires;
          }
        }
      }
    }

    foreach ($dependency_badorder as $parent => $items) {
      $dnames = array();
      foreach ($items as $item) {
        $dnames[] = $plugins[$item]->info['name'];
      }
      $error = new stdClass();
      $error->type    = 'dependency bad order';
      $error->phase   = 'build';
      $error->message = 'Theme plug-in <em>!this.name</em> depends on <em>!dependency.names</em>. This is not a valid use of dependencies. Plug-ins hosted in base themes are not allowed to point to plug-ins hosted in a sub-theme.';
      $error->strings['!this.name']        = $plugins[$parent]->info['name'];
      $error->strings['!dependency.names'] = implode(', ', $dnames);
      $errors[$parent][] = $error;
      if (isset($plugins[$parent])) {
        unset($plugins[$parent]);
      }
    }
    foreach ($dependency_errors as $child => $items) {
      $error = new stdClass();
      $error->type    = 'dependency error';
      $error->phase   = 'build';
      $error->message = 'Theme plug-in <em>!this.name</em> depends on <em>!dependency.names</em> which contains errors.';
      $error->strings['!this.name']        = $plugins[$child]->info['name'];
      $error->strings['!dependency.names'] = implode(', ', $dnames);
      $errors[$child][] = $error;
      if (isset($plugins[$child])) {
        unset($plugins[$child]);
      }
    }
    foreach ($dependency_missing as $child => $items) {
      $error = new stdClass();
      $error->type    = 'dependency missing';
      $error->phase   = 'build';
      $error->message = 'Theme plug-in <em>!this.name</em> depends on <code>!dependency.keys</code> which cannot be found.';
      $error->strings['!this.name']       = $plugins[$child]->info['name'];
      $error->strings['!dependency.keys'] = implode(', ', $items);
      $errors[$child][] = $error;
      if (isset($plugins[$child])) {
        unset($plugins[$child]);
      }
    }

    // Expose dependency.
    foreach ($plugins as $plugin) {
      $plugin->info['required_by'] = array_keys($plugin->required_by);
      $plugin->info['requires']    = array_keys($plugin->requires);
    }
  }

  /* Build Activation List ------------------------------------------------ */

  // Build activation list.
  $activate_list = array();

  // Get theme enabled plug-ins.
  $theme_activated = $locked = array();
  $theme_activated_module_deactivated = array();

  foreach (hex_active_themes() as $theme) {
    $activate_keys = array();

    // Gather saved states from theme settings.
    if ($theme === hex_active_theme()) {
      foreach ($plugins as $key => $plugin) {
        if (hex_themeapi_setting("$key.enable", 0, $theme)) {
          $activate_keys[] = $key;
        }
      }
    }

    // .info activations will always be enabled.
    $host_enabled  = hex_theme_info(hex_active_theme() . '.plugins', array(), $theme);
    $theme_enabled = hex_theme_info('theme.plugins', array(), $theme);
    $current_locks = array_merge($host_enabled, $theme_enabled);
    $activate_keys = array_unique(array_merge($activate_keys, $host_enabled, $theme_enabled));

    foreach ($activate_keys as $key) {
      if (isset($plugins[$key])) {
        if (in_array($key, $activate_list)) {
          // Preven duplicate entries.
          continue;
        }

        $plugin = $plugins[$key];
        $missing_modules = array_diff($plugin->info['module.dependencies'], module_list());

        if (!$missing_modules) {
          $activate_list[] = $key;
          $theme_activated[$key] = $theme;
          if (in_array($key, $current_locks)) {
            $locked[] = $key;
          }
        }
        // If a dependent module was deactivated.
        elseif (isset($persist_set[$key])) {
          $theme_activated_module_deactivated[$key] = $missing_modules;
        }
      }
      elseif (!isset($errors[$key])) {
        $error = new stdClass();
        $error->type    = 'missing';
        $error->phase   = 'activate';
        $error->message = 'Theme plug-in <code>!this</code> does not exist. The plug-in was either removed or the .info setting for <em>!enabler.name</em> was not set properly.';
        $error->strings['!this']         = $key;
        $error->strings['!enabler.name'] = hex_theme_info('name', 'Untitled', $theme);
        $errors[$activate_key][] = $error;
      }
    }
  }

  // Auto load compatibility plug-ins.
  $module_activated = array();
  foreach ($plugins as $key => $plugin) {
    if ($plugin->info['plugin.type'] === 'compatibility'
    && !empty($plugin->info['module.dependencies'])
    && !array_diff($plugin->info['module.dependencies'], module_list())) {
      $activate_list[] = $key;
      $module_activated[$key] = $plugin->info['module.dependencies'];
      $locked[] = $key;
    }
  }

  // Create an associative array with weights as values.
  $activate_list = array_flip(array_values(array_unique($activate_list)));

  // Add in dependencies and sort weights.
  $dependency_list = array();
  while(list($key) = each($activate_list)) {
    // Set sorting weights.
    $activate_list[$key] = $plugins[$key]->sort;
    // Set dependencies.
    foreach (array_keys($plugins[$key]->requires) as $dependent) {
      $activate_list[$dependent] = 0;
      $dependency_list[$key][] = $plugins[$dependent]->info['name'];
      $locked[] = $dependent;
    }
  }

  /**
   * Set Notifications -------------------------------------------------------
   * Notifications displayed in a specific order:
   *
   *  - Theme enabled
   *  - Auto enabled (for module compatibility plug-ins)
   *  - Dependency enabled
   *  - All disabled
   *  - Warnings
   *  - Errors
   */

  // Status messages for theme enabled plug-ins.
  foreach ($theme_activated as $key => $theme) {
    // Not active previously. Give message.
    if (!isset($persist_set[$key])) {
      $t = array('!this.name'  => $plugins[$key]->info['name'],
                 '!host.name' => hex_theme_info('name', 'Untitled', $theme));
      $t += array('!this.description.status' => t($plugins[$key]->info['description.status'], $t));

      hex_notify('Theme plug-in <em>!this.name</em> was enabled by <em>!host.name</em>. !this.description.status', $t, 'status');
    }
  }

  // Status messages for compatibility plug-ins.
  foreach ($module_activated as $key => $modules) {
    // Not active previously. Give message.
    if (!isset($persist_set[$key])) {
      // Parse module.info to get at the human readable data.
      $t = array();
      foreach ($modules as $module) {
        $module_info = drupal_parse_info_file(hex_path('module', $module) . '/' . $module . '.info');
        $t += array("!$module"      => $module,
                    "!$module.name" => $module_info['name']);
      }
      $t += array('!host'      => $plugins[$key]->hosts[0],
                  '!host.name' => hex_theme_info('name', 'Untitled', $plugins[$key]->hosts[0]));
      $t += array('!this'                    => $key,
                  '!this.name'               => $plugins[$key]->info['name'],
                  '!this.description.status' => t($plugins[$key]->info['description.status'], $t_keys));

      hex_notify('Theme plug-in <em>!this.name</em> was automatically enabled. !this.description.status.', $t, 'status');
    }
  }

  // Status messages for plug-in dependencies.
  foreach ($dependency_list as $key => $dependents) {
    if (!isset($persist_set[$key]) && (isset($theme_activated[$key]) || isset($module_activated[$key]))) {
      $t = array('!this.name'         => $plugins[$key]->info['name'],
                 '!dependency_plural' => format_plural(count($dependents), '1 dependency', '@count dependencies'),
                 '!dependency_names'  => implode(', ', $dependents));

      hex_notify('!dependency_plural enabled for <em>!this.name</em>: <em>!dependency_names</em>', $t, 'status');
    }
  }

  // Status messages for disabled plug-ins.
  $disabled_list = array();
  foreach (array_reverse(array_diff(array_keys($persist_set), array_keys($activate_list))) as $key) {
    if (isset($theme_activated_module_deactivated[$key])) {
      $t = array('!this.name'       => $plugins[$key]->info['name'],
                 '!required_plural' => format_plural(count($theme_activated_module_deactivated[$key]), 
                                                     'a required module',
                                                     '@count required modules'));

      hex_notify('Disabled theme plug-in <em>!this.name</em> due to !required_plural being disabled.', $t, 'status');
    }
    elseif (isset($plugins[$key])) {
      $disabled_list[] = $plugins[$key]->info['name'];
    }
  }
  if ($disabled_list) {
    $t = array('!plugin_plural' => format_plural(count($disabled_list), '1 theme plug-in', '@count theme plug-ins'),
               '!plugin.names'  => implode(', ', $disabled_list));

    hex_notify('Disabled !plugin_plural: <em>!plugin.names</em>.', $t, 'status');
  }

  foreach ($warnings as $key => $items) {
    foreach ($items as $item) {
      hex_notify($item->message, $item->strings, 'warning');
    }
  }
  // Activation errors.
  foreach ($errors as $key => $items) {
    foreach ($items as $i => $item) {
      if ($item->phase === 'activate') {
        hex_notify($item->message, $item->strings, 'error');
        unset($errors[$key][$i]);
      }
    }
  }
  // Other errors.
  foreach ($errors as $key => $items) {
    foreach ($items as $item) {
      hex_notify($item->message, $item->strings, 'error');
    }
  }

  /**
   * Final Build -------------------------------------------------------------
   * Build the following sets:
   * @see hex_plugin_list() For a full explanation.
   *
   *  - all
   *  - all.active
   *  - all.active.locked
   *  - all.active.ENABLER
   *  - all.active.ENABLER.locked
   *  - all.TYPE
   *  - all.TYPE.active
   *  - all.TYPE.active.locked
   *  - all.TYPE.active.ENABLER
   *  - all.TYPE.active.ENABLER.locked
   *
   *  - HOST
   *  - HOST.active
   *  - HOST.active.locked
   *  - HOST.active.ENABLER
   *  - HOST.active.ENABLER.locked
   *  - HOST.TYPE
   *  - HOST.TYPE.active
   *  - HOST.TYPE.active.locked
   *  - HOST.TYPE.active.ENABLER
   *  - HOST.TYPE.active.ENABLER.locked
   */

  // Sort the plug-in list by pre-calculated weights.
  arsort($activate_list);

  $lists = array();
  foreach ($plugins as $key => $plugin) {
    // $plugin->host[0] is the theme where the plug-in resides.
    $host = $plugin->hosts[0];
    $type = $plugin->info['plugin.type'];

    $lists['all'][$key]         = $key;
    $lists[$host][$key]         = $key;
    $lists["all.$type"][$key]   = $key;
    $lists["$host.$type"][$key] = $key;
  }

  foreach (array_keys($activate_list) as $key) {
    $host = $plugins[$key]->hosts[0];
    $type = $plugins[$key]->info['plugin.type'];

    $activate_levels = array('active');
    if (in_array($key, $locked)) {
      $activate_levels[] = 'active.locked';
    }
    foreach ($activate_levels as $activate) {
      $lists["all.$activate"][$key]         = $key;
      $lists["$host.$activate"][$key]       = $key;
      $lists["all.$type.$activate"][$key]   = $key;
      $lists["$host.$type.$activate"][$key] = $key;

      $enabled_by = array();
      if (isset($theme_activated[$key])) {
        $enabled_by[] = $theme_activated[$key];
      }
      if (isset($module_activated[$key])) {
        $enabled_by[] = implode('+', $module_activated[$key]);
        if (count($module_activated[$key]) > 1) {
          $enabled_by = array_merge($enabled_by, $module_activated[$key]);
        }
      }
      foreach ($enabled_by as $enabler) {
        $lists["all.$activate.$enabler"][$key]         = $key;
        $lists["$host.$activate.$enabler"][$key]       = $key;
        $lists["all.$type.$activate.$enabler"][$key]   = $key;
        $lists["$host.$type.$activate.$enabler"][$key] = $key;
      }
    }
  }

  variable_set($persist_key, $activate_list);

  return array(
    'plugins' => $plugins,
    'lists'   => $lists,
  );
}
