<?php

/**
 * Returns the path to the most relevant file by passing in its name.
 *
 * The file returned will be closest to the active theme. This is important to
 * understand when the active theme has a parent or multiple parents and/or
 * plug-ins. It will fallback in this order:
 *
 * - active theme
 * - plug-in hosted by the active theme
 * - parent theme
 * - plug-in hosted by the parent theme
 * - etc...
 *
 * This also depends on the group and scope of the defined file extensions
 * and patterns and how this function is called. Groups are defined from .info
 * or .pinfo files when defining file extensions and patterns.
 *
 *  file.extensions[GROUP_NAME] = .ext
 *
 *  file.patterns[GROUP_NAME]   = /^regex$/
 *
 * There is a 'general' group for files defined without an explicit group name.
 *
 *  file.extensions[] = .css
 *  file.extensions[] = .js
 *
 *  file.patterns[]   = /^regex$/
 *
 * The scope defines where the search occurs and it must be prefixed before
 * the group separated with a colon. Multiple scopes can be set delimited by a
 * vertical bar. Valid values are "all", "theme", "plugin", a specific theme
 * key or plugin key.
 *
 *  file.extensions[SCOPE:GROUP_NAME]         = .ext
 *
 *  file.patterns[SCOPE:GROUP_NAME]           = /^regex$/
 *  file.patterns[SCOPE1|SCOPE2:GROUP_NAME]   = /^regex$/
 *  file.patterns[theme|plugins:color_styles] = /^[[:alpha:]][[:alnum:]._-]+\.colors\.css$/
 *
 * If the scope and group is not defined, the scope is automatically set to
 * "theme" or "plugin" and part of the "general" group depending on where it
 * was set. Set from the .pinfo file (plug-in) will set it to "plugin" for
 * example.
 *
 * File patterns can also be set from a hook supplied by Hexagon. The same
 * rules apply as setting them from info files except that it only works with
 * regex patterns:
 *
 *  function [THEME_NAME|PLUGIN_NAME]_search_patterns($existing) {
 *    return array(
 *      'SCOPE|GROUP_NAME' => '/^regex$/',
 *    );
 *  }
 * 
 * Note: If a file or directory is prefixed with a ~tilde, it will be ignored.
 * This may be useful when you need to disable the file or group of files
 * temporarily. Also keep in mind that the path is cached. The cache monitors
 * the theme registry so if it is cleared, so will the files cache. There's
 * a theme setting to force all theme related caches to clear. See the theme
 * settings page or add this into the active theme's .info file.
 *
 *  settings[development_mode] = 1
 *  settings[development_mode:disable_theme_caching] = 1
 *
 * @param $name
 *  The file name.
 * @param $group
 *  The group name. Use 'general' for files not defined with a specific group.
 * @param $scope
 *  Can be 'theme', 'plugin' or 'all'. It can also be a plug-in or theme name.
 *  whatever "owns" the file. File discovery depends on how the scope is
 *  defined in the group prefix and it must be accessed the same way with
 *  'all' being the exception since it looks everywhere.
 */
function hex_path_to_file($name, $group = 'general', $scope = 'all') {
  $files = hex_get_files($group, $scope);
  return !empty($files[$name]) ? end($files[$name])->uri : NULL;
}

/**
 * Returns an array of file paths for a given name.
 *
 * @see hex_path_to_file()
 */
function hex_path_to_file_multiple($name, $group = 'general', $scope = 'all') {
  $files = hex_get_files($group, $scope);
  $file_multi = array();
  if (!empty($files[$name])) {
    foreach ($files[$name] as $file) {
      $file_multi[] = $file->uri;
    }
  }
  return $file_multi;
}

/**
 * Returns an array of file objects.
 *
 * @see hex_path_to_file()
 */
function hex_get_files($group, $scope) {
  $files = _hex_cache('files');
  return isset($files[$scope][$group]) ? $files[$scope][$group] : array();
}

/**
 * Simply returns a list of all defined groups set from file patterns.
 *
 * @see hex_path_to_file()
 */
function hex_list_file_groups($scope = 'all') {
  $files = _hex_cache('files');
  return !empty($files[$scope]) ? array_keys($files[$scope]) : array();
}
