<?php

// Plugin group weights for drupal_add_css() and drupal_add_js().
define('CSS_PLUGIN', 200);
define('JS_PLUGIN',  200);

/**
 * Get info data specific to a given plug-in. The information will be pulled
 * from the plug-in's .pinfo file.
 * 
 * @param $pinfo_key
 *  (required) The .pinfo key in the plug-in.
 * @param $default
 *  (required) Default value if none is found.
 * @param $plugin
 *  (required) The plug-in to target.
 */
function hex_plugin_info($pinfo_key, $default, $plugin) {

  $data = _hex_cache('plugins');
  if (isset($data['plugins'][$plugin]->info[$pinfo_key])) {
    $plugin_info = $data['plugins'][$plugin]->info[$pinfo_key];
  }

  // Pass through defauts.
  if (!isset($plugin_info)) {
    $plugin_info = $default;
  }

  return $plugin_info;
}

/**
 * Check to see if a plug-in exists and is active.
 *
 * @see hex_plugin_list()
 *  More details on the $set parameter.
 *
 * @param $plugin
 *  The plugin key
 * @param $set
 *  Checks for a specific set. Defaults to all active plug-ins.
 */
function hex_plugin_exists($plugin, $set = 'all.active') {
  return in_array($plugin, hex_plugin_list($set));
}

/**
 * Return the path to a given plug-in.
 * 
 * If there are multiple copies of a plug-in with the same name that exists
 * across base themes and sub-themes, the returned path will point to the most
 * active, i.e., the active sub-theme or the sub-theme closest to the current
 * theme hosting the plug-in.
 * 
 * @param $plugin
 *  (required) The plug-in name.
 */
function hex_plugin_path($plugin) {
  $data = _hex_cache('plugins');
  return isset($data['plugins'][$plugin]) ? $data['plugins'][$plugin]->directory : NULL;
}

/**
 * Returns a list of plug-in's.
 *
 * The set can be used to filter the output. Use ".active" for enabled
 * plug-ins. The ".ENABLER" is what enabled the plug-in. It's typically the
 * theme but it can also be a module when the plug-in type is a compatibility
 * plug-in. The ".TYPE" is determined by the .pinfo "plugin.type" setting. The
 * "HOST" is the theme where the plug-in is hosted and ".active.locked" are
 * for plug-ins that cannot be disabled through the theme settings form
 * because it is a required plug-in.
 *
 *  - all
 *  - all.active
 *  - all.active.locked
 *  - all.active.ENABLER
 *  - all.active.ENABLER.locked
 *  - all.TYPE
 *  - all.TYPE.active
 *  - all.TYPE.active.locked
 *  - all.TYPE.active.ENABLER
 *  - all.TYPE.active.ENABLER.locked
 *
 *  - HOST
 *  - HOST.active
 *  - HOST.active.locked
 *  - HOST.active.ENABLER
 *  - HOST.active.ENABLER.locked
 *  - HOST.TYPE
 *  - HOST.TYPE.active
 *  - HOST.TYPE.active.locked
 *  - HOST.TYPE.active.ENABLER
 *  - HOST.TYPE.active.ENABLER.locked
 *
 * When a plug-in requires more than one module and its type is a
 * compatibility plug-in, the ENABLER will be set in multiple forms. Keep in
 * mind that compatibility plug-ins are automatically enabled triggered by the
 * state of enabled modules. Example plug-in .pinfo file:
 *
 *  plugin.type = compatibility
 *
 *  module.dependencies[] = overlay
 *  module.dependencies[] = toolbar
 * 
 * Here is a partial list of possible $set's which would list the plug-in
 * when active:
 *
 *  - all.active.overlay+toolbar
 *  - all.active.overlay
 *  - all.active.toolbar
 *  - all.compatibility
 *  - all.compatibility.active.overlay+toolbar
 *  - all.compatibility.active.overlay
 *  - all.compatibility.active.toolbar
 *
 * In all other cases, ENABLER will be the theme that enabled the plug-in.
 *
 * @param $set
 *  Used to filter the output. Note that none of the sets will include
 *  plug-in's hosted in disabled themes.
 */
function hex_plugin_list($set = NULL) {
  $data = _hex_cache('plugins');
  if (!isset($set)) {
    $set = 'all.active';
  }
  return isset($data['lists'][$set]) ? $data['lists'][$set] : array();
}

/**
 * Returns a list of all available sets.
 *
 * @see hex_plugin_lists()
 */
function hex_plugin_sets() {
  $data = _hex_cache('plugins');
  return array_keys($data['lists']);
}